select
  an.numero
  , (case when ((ultlac.cdlactacao is not null) and (penultlac.cdlactacao is not null))
            then (ultlac.dtinicio - penultlac.dtinicio) else null end) as ult_iel
  , (case when ((ultpar.cdreproducao is not null) and (penultpar.cdreproducao is not null))
            then (ultpar.data - penultpar.data) else null end) as ult_iep
  , (case ultcob.condicaoia when 'I' then 'Protocolo hormonal'
                        when 'N' then 'Cio natural'
                        when 'P' then 'Prostaglandina' else null end) as ult_condicao_ia
  , ((select count(*) from reproducao re where re.cdtiporeproducao = 7 and re.cdanimal = an.cdanimal) + Coalesce(an.numpartoentrada, 0)) as total_partos
  , tp.descricao as tipo_ult_parto
  , (case catrep.cdcategoria when 1 then 'Insemin./Cobert.'
                        when 2 then 'Implantada'
                        when 3 then 'Gestante'
                        when 4 then 'Vazia' else null end) as sit_reprodutiva
  , (case when (ultlacenc.cdlactacao is not null)
            then (select producaototal from sp_calculaproducaototal(ultlacenc.cdlactacao, ultlacenc.dtfim)) else null end) as total_ult_lac
  , (case when ((ultlac.cdlactacao is not null) and (ultlac.dtfim is null))
            then (select producaototal from sp_calculaproducaototal(ultlac.cdlactacao, current_date)) else null end) as total_lac_atual
  , (case when ((ultpar.cdreproducao is not null) and (ultpar.retencaoplacenta = 1)) then 'Sim' else null end) as retencao_placenta
  , ((select count(*) from lactacao re where re.cdanimal = an.cdanimal) + coalesce(an.numpartoentrada, 0)) as total_lactacoes
  , (select count(*) from reproducao re where re.cdanimal = an.cdanimal and re.cdtiporeproducao in (1,2,3) and re.data < ultpar.data
        and re.data > coalesce(penultpar.data, '1900-01-01')) as num_tent_ult_par
  , (case when ((ultlac.cdlactacao is not null) and (ultlac.dtfim is null)) then
                (select count(*) from aplicacaoproduto re join produto repr on repr.cdproduto = re.cdproduto where re.cdanimal = ultlac.cdanimal
                        and re.dtaplicacaoproduto >= ultlac.dtinicio and repr.cdproduto in
                        (select cdproduto from produtoprincipioativo where cdprincipioativo in (234,235)))
            else null end) as num_bst_lac_atual
  , an.dtnascimento as data_nascimento
  , (case when ((ultpar.cdreproducao is not null) and (ultev.dtpartoprovavel is not null)) then (ultev.dtpartoprovavel - ultpar.data) else null end) as IEP_projetado
  , ((current_date - an.dtnascimento)/30.5) as idade_meses
  , (case when (priparto.cdreproducao is not null) then ((priparto.data - an.dtnascimento) / 30.5) else null end) as idade_prim_parto_meses
  , (select grausangue from sp_calculagrausangue(an.cdanimal)) as raca
  , (case when ((ultlac.cdlactacao is not null) and (ultlac.dtfim is null) and (penultlac.dtfim is not null)) then
                    (ultlac.dtinicio - penultlac.dtfim)
          else null end) as ult_periodo_seco
  , (case when (ultlac.dtfim is not null) then (ultlac.dtfim - ultlac.dtinicio)
            when ((ultlac.dtfim is null) and (penultlac.cdlactacao is not null)) then (penultlac.dtfim - penultlac.dtinicio)
            else null end) as duracao_ult_lac_enc
  , (case when ((ultlac.cdlactacao is not null) and (ultlac.dtfim is null)) then (coalesce(an.dtbaixa, current_date) - ultlac.dtinicio) else null end) as del
  , (case when (ultcio.cdreproducao is not null) then (current_date - ultcio.data) else null end) as dias_apos_cio
  , (case when (catrep.cdcategoria in (1,2)) then (current_date - ultcob.data) else null end) as dias_ia_te_cob
  , coalesce(ultlac.dtfim, penultlac.dtfim) as dt_ult_secagem
  , (case when (catrep.cdcategoria = 3) then (ultev.dtpartoprovavel) else null end) as dt_prox_parto
  , ultcob.data as data_ult_ia_te_cob
  , (case when (pricob.cdreproducao is not null) then (pricob.data - ultpar.data) else null end) as int_parto_pri_tent
from
  animal an
  left join categoria catrep on catrep.cdcategoria =
    (select creprodutiva from sp_categoriaanimal(current_date, 'cdanimal = ' || an.cdanimal))
  left join lactacao ultlac on ultlac.cdlactacao =
    (select first 1 re.cdlactacao from lactacao re where re.cdanimal = an.cdanimal order by re.dtinicio desc)
  left join lactacao penultlac on penultlac.cdlactacao =
    (select first 1 re.cdlactacao from lactacao re where re.cdanimal = ultlac.cdanimal and re.dtinicio < ultlac.dtinicio order by re.dtinicio desc)
  left join reproducao ultpar on ultpar.cdreproducao =
    (select first 1 re.cdreproducao from reproducao re where re.cdanimal = an.cdanimal and re.cdtiporeproducao = 7 order by re.data desc)
  left join tipoparto tp on tp.cdtipoparto = ultpar.cdtipoparto
  left join reproducao penultpar on penultpar.cdreproducao =
    (select first 1 re.cdreproducao from reproducao re where re.cdanimal = ultpar.cdanimal and re.cdtiporeproducao = 7 and
        re.data < ultpar.data order by re.data desc)
  left join reproducao ultcob on ultcob.cdreproducao =
    (select first 1 re.cdreproducao from reproducao re where re.cdanimal = an.cdanimal and re.data > coalesce(ultpar.data, '1900-01-01')
        and re.cdtiporeproducao in (1,2,3) order by re.data desc)
  left join lactacao ultlacenc on ultlacenc.cdlactacao =
    (select first 1 re.cdlactacao from lactacao re where re.cdanimal = an.cdanimal and re.dtfim is not null order by re.dtinicio desc)
  left join reproducao ultev on ultev.cdreproducao =
    (select first 1 re.cdreproducao from reproducao re where re.cdanimal = an.cdanimal and
            ((re.cdtiporeproducao in (1,2,3,7)) or (re.cio in (1,2)) or (re.diagnostico is not null)) order by re.data desc)
  left join reproducao priparto on priparto.cdreproducao =
    (select first 1 re.cdreproducao from reproducao re where re.cdanimal = an.cdanimal and re.cdtiporeproducao = 7 order by re.data asc)
  left join reproducao ultcio on ultcio.cdreproducao =
    (select first 1 re.cdreproducao from reproducao re where re.cdanimal = an.cdanimal and re.cio in (1,2)
        and re.data > coalesce(ultpar.data, '1900-01-01') order by re.data desc)
  left join reproducao pricob on pricob.cdreproducao =
    (select first 1 re.cdreproducao from reproducao re where re.cdanimal = ultpar.cdanimal and re.data > ultpar.data and re.cdtiporeproducao in (1,2,3)
        order by re.data asc)
where
  an.tipoanimal = 'A'
  and an.dtbaixa is null
  and an.animalrebanho = 1
  and an.cdcategoria in (6,7)
order by
  an.numordenado
